{..............................................................................}
{Version 1, CablePullsheet_v1.pas}
{ Summary Using data extracted from a focused Altium schematic, this script    }
{ generates a pull list in a .csv format for importing into  }
{ Microsoft Excel.  Written by D. Sigg and R. Abbott 2017}             }
{..............................................................................}

{..............................................................................}
Var
    TargetFileName : ShortString;
    NetList        : TStringList;
{..............................................................................}

{..............................................................................}
{ Replace up to two '-' with comma in a designator name }
Function SeparateDesignator (Const d : String) : String;
Var
    Position : Integer;
    ret : String;
Begin
    ret := d;
    Position := LastDelimiter ('-', ret);
    If Position <> 0 Then ret[Position] := ','
    Else ret := ret + ',';
    Position := LastDelimiter ('-', ret);
    If Position <> 0 Then ret[Position] := ','
    Else ret := ret + ',';
    Result := ret;
End;
{..............................................................................}

{..............................................................................}
{ Separate pin name into label and connector }
Function SeparatePinName (Const PinName : String,
                   Var LabelName : String,
                   Var Connector : String) : Bool;
Var
    Position : Integer;
Begin
    Position := LastDelimiter ('/', PinName);
    If Position = 0 Then
    Begin
       LabelName := PinName;
       Connector := '';
    End
    Else Begin
       LabelName := AnsiLeftStr (PinName, Position - 1);
       Connector := AnsiRightStr (PinName, Length(PinName) - Position);
    End;
    Result := True;
End;
{..............................................................................}

{..............................................................................}
{ Replace up to two '-' with comma in a designator name }
Function CableLookup (Const Connector1, Connector2, CableLength : String) : String;
Var
    Position : Integer;
    dcc : String;
    conn1, conn2 : String;
Begin
    dcc := '';
    If CompareText (Connector1,Connector2) < 0 Then
    Begin
       conn1 := Connector1;
       conn2 := Connector2;
    End
    Else Begin
       conn1 := Connector2;
       conn2 := Connector1;
    End;
    If SameString (conn1, 'TNC', False) And
       SameString (conn2, 'TNC', False) Then dcc := 'E1800001'
    Else If SameString (conn1, 'SMA', False) And
       SameString (conn2, 'TNC', False) Then dcc := 'E1800002'
    Else If SameString (conn1, 'SMA', False) And
       SameString (conn2, 'SMA', False) Then dcc := 'E1800003';
    { add more as apprpriate }
    Result := dcc;
End;
{..............................................................................}

{..............................................................................}
Function GetPartParameters (Part : IPart,
                   Var Comment : String,
                   Var DCC : String,
                   Var System : String,
                   Var Rack : String,
                   Var Slot : String,
                   Var Height : String) : Boolean;
Var
   I       : Integer;
   Param   : IParameter;
   ParamN  : String;
   ParamV  : String;

Begin
   Result := False;
   Comment := '';
   DCC := '';
   System := '';
   Rack := '';
   Slot := '';
   Height := '';
   If Part = Nil Then Exit;

   For I := 0 to Part.DM_ParameterCount - 1 Do
   Begin
      Param := Part.DM_Parameters(i);
      ParamN := Param.DM_Name();
      ParamV := Param.DM_Value();
      If SameString (ParamN, 'Comment', False) Then Comment := ParamV
      Else If SameString (ParamN, 'DCC', False) Then DCC := ParamV
      Else If SameString (ParamN, 'System', False) Then System := ParamV
      Else If SameString (ParamN, 'Rack', False) Then Rack := ParamV
      Else If SameString (ParamN, 'Slot', False) Then Slot := ParamV
      Else If SameString (ParamN, 'Height', False) Then Height := ParamV
   End;
   Result := True;
End;
{..............................................................................}

{..............................................................................}
Procedure WriteNet(Net : INet);
Var
   I                    : Integer;
   Param                : IParameter;
   ParamN               : String;
   ParamV               : String;
   Pin                  : IPin;
   Csv                  : String;

   NetName              : String;
   CableLength          : String;
   CableDCC             : String;
   Comment1, Comment2   : String;
   DCC1, DCC2           : String;
   System1, System2     : String;
   Rack1, Rack2         : String;
   Slot1, Slot2         : String;
   Height1, Height2     : String;
   Panel1,Panel2        : String;
   Connector1,Connector2: String;
   Label1,Label2        : String;
Begin
    If Net.DM_PinCount >= 2 Then
    Begin
        NetName := Net.DM_CalculatedNetName;
        { Get cable length from net parameter }
        CableLength := '';
        CableDCC := '';
        For I := 0 To Net.DM_ParameterCount - 1 Do
        Begin
            Param := Net.DM_Parameters(i);
            ParamN := Param.DM_Name();
            ParamV := Param.DM_Value();
            If SameString (ParamN, 'Length', False) Then CableLength := ParamV
            Else If SameString (ParamN, 'DCC', False) Then CableDCC := ParamV;
        End;
        { Look for part parameters associated with pin 0 }
        Pin := Net.DM_Pins(0);
        If Not GetPartParameters (Pin.DM_Part, Comment1, DCC1,
                                  System1,Rack1,Slot1,Height1) Then Exit;
        { Look for panel location, connector & label }
        Label1 := Pin.DM_PinName;
        Connector1 := Pin.DM_PinNumber;
        Panel1 := 'Front';
        If (Pin.DM_Electrical = eElectricIO) Or
           (Pin.DM_Electrical = eElectricOutput) Or
           (Pin.DM_Electrical = eElectricInput) Then
        Begin
            Panel1 := 'Rear';
        End;
        { loop over remaining pins }
        For I := 1 To Net.DM_PinCount - 1 Do
        Begin
            { Look for part parameters associated with pin i }
            Pin := Net.DM_Pins(I);
            If Not GetPartParameters (Pin.DM_Part, Comment2, DCC2,
                                  System2,Rack2,Slot2,Height2) Then Exit;
            { Look for panel location, connector & label }
            Label2 := Pin.DM_PinName;
            Connector2 := Pin.DM_PinNumber;
            Panel2 := 'Front';
            If (Pin.DM_Electrical = eElectricIO) or
               (Pin.DM_Electrical = eElectricOutput) Or
               (Pin.DM_Electrical = eElectricInput) Then
            Begin
                Panel2 := 'Rear';
            End;
            { Look up cable dcc }
            If (CableDCC = '') And (Net.DM_PinCount = 2) Then
                CableDCC := CableLookup (Connector1, Connector2, CableLength);
            Begin
            End;
            { csv string }
            csv := NetName + ',' +
               CableDCC + ',' +
               CableLength + ',' +
               Comment1 + ',' +
               DCC1 + ',' +
               System1 + ',' +
               Rack1 + ',' +
               Slot1 + ',' +
               Label1 + ',' +
               Panel1 + ',' +
               Connector1 + ',' +
               Comment2 + ',' +
               DCC2 + ',' +
               System2 + ',' +
               Rack2 + ',' +
               Slot2 + ',' +
               Label2 + ',' +
               Panel2 + ',' +
               Connector2;
            NetList.Add (csv);
        End;
        {
        NetList.Add(Net.DM_CalculatedNetName + ',' +
          SeparateDesignator(Net.DM_Pins(0).DM_PhysicalPartDesignator) + ',' +
          Net.DM_Pins(0).DM_PinNumber + ',' +
          SeparateDesignator(Net.DM_Pins(1).DM_PhysicalPartDesignator) + ',' +
          Net.DM_Pins(1).DM_PinNumber
        );
        }
    End;
End;
{..............................................................................}

{..............................................................................}
Procedure CreateAllDirectories(Dir : String);
Var
    ldir  : Integer;
    fpath : String;
Begin
    ldir := Length(Dir);
    If ldir = 0 Then Exit;

    If Dir[ldir] = '\' Then
        Delete(Dir, ldir, 1);

    fpath := ExtractFilePath(Dir);

    If (Length(Dir) < 3) Or DirectoryExists(Dir) Or (fPath = Dir) Then Exit;

    CreateAllDirectories(fPath);
    CreateDir(Dir);
End;
{..............................................................................}

{..............................................................................}
Procedure Generate(Const Project : IProject; Const DocumentPath, TargetFolder : WideString);// Version : Integer);
Var
    i              : Integer;
    Document       : IDocument;
    AFullPath      : WideString;
    Msg            : WideString;
Begin
    TargetFileName := '';
    Document := Project.DM_GetDocumentFromPath(DocumentPath);

    If Project.DM_DocumentFlattened <> Nil Then
       AFullPath := Project.DM_ProjectFullPath
    Else
    If Document <> Nil Then
       AFullPath := Document.DM_FullPath
    Else {Use Free Documents Project}
       AFullPath := Project.DM_ProjectFullPath;

    TargetFileName := Project.DM_GetOutputPath + '\' + ExtractFileNameFromPath(AFullPath) + '_CableList.csv';

    If (Project.DM_DocumentFlattened = Nil) And (Document = Nil) Then Exit;

    If (Client <> Nil) And (Client.GUIManager <> Nil) Then
    Begin
        Msg := 'Writing Protel NetList to ' + ExtractFileName(TargetFileName) + '...';
        Client.GUIManager.StatusBar_SetState(0, Msg);
    End;

    CreateAllDirectories(ExtractFilePath(TargetFileName));
    // Check if project has been flattened...
    Document := Project.DM_DocumentFlattened;
    If Document = Nil Then
       Document := Project.DM_GetDocumentFromPath(DocumentPath);

    // Generate net information
    NetList.Add('');//Generate a row for spacing
    //NetList.Add('Cable Length (Ft),,From Location,Connector Type,Orientation,From Text,To Location,Connector Type,Orientation,To Text');//Label Columns
    NetList.Add('Cable Number,Cable Info,Cable Length (Ft),From Description,From D-number,From Subsystem,From Rack,From U-height,From Connector Text,From Connector Location,From Connector Type, To Description,To D-number,To Subsystem,To Rack,To U-height,To Connector Text,To Connector Location,To Connector Type');//Label Columns
    For i := 0 To Document.DM_NetCount - 1 Do
             WriteNet(Document.DM_Nets(i));
    NetList.SaveToFile(TargetFileName);

    // Sets the Editor Type and put in the Generated folder.
    VFS_SetFileEditorName(TargetFileName,'ProtelNetlist');
    Project.DM_AddGeneratedDocument(TargetFileName);
End;
{..............................................................................}

{..............................................................................}
Procedure CablePullsheet;
Var
    WS      : IWorkspace;
    Prj     : IProject;
Begin
    WS  := GetWorkspace;
    If WS = Nil Then Exit;

    Prj := WS.DM_FocusedProject;
    If Prj = Nil Then Exit;

    // Compile the project to fetch the connectivity
    // information for the design.
    Prj.DM_Compile;

    // Create a TStringList to store component and net data.
    NetList := TStringList.Create;

    // Generate Netlist.
    Generate(Prj, Prj.DM_ProjectFullPath, ExtractFilePath(Prj.DM_ProjectFullPath));
    NetList.Free;
End;
{..............................................................................}

{..............................................................................}
End.

